//
//  NSArray+GHLCrashGuard.m
//  GHLCrashGuard
//
//  Created by 与佳期 on 2019/7/11.
//

#import "NSArray+GHLCrashGuard.h"
#import "JRSwizzle.h"

@implementation NSArray (GHLCrashGuard)

+ (void)load {
    
    // [NSArray alloc]
    [NSClassFromString(@"__NSPlaceholderArray") jr_swizzleMethod:@selector(initWithObjects:count:) withMethod:@selector(initWithObjects_guard:count:) error:nil];
    // @[]
    [NSClassFromString(@"__NSArray0") jr_swizzleMethod:@selector(objectAtIndex:) withMethod:@selector(guard_objectAtIndex:) error:nil];
    [NSClassFromString(@"__NSArray0") jr_swizzleMethod:@selector(arrayByAddingObject:) withMethod:@selector(guard_arrayByAddingObject:) error:nil];
    // @[@1]
    [NSClassFromString(@"__NSSingleObjectArrayI") jr_swizzleMethod:@selector(objectAtIndex:) withMethod:@selector(guard_objectAtIndex:) error:nil];
    [NSClassFromString(@"__NSSingleObjectArrayI") jr_swizzleMethod:@selector(arrayByAddingObject:) withMethod:@selector(guard_arrayByAddingObject:) error:nil];
    // @[@1, @2]
    [NSClassFromString(@"__NSArrayI") jr_swizzleMethod:@selector(objectAtIndex:) withMethod:@selector(guard_objectAtIndex:) error:nil];
    [NSClassFromString(@"__NSArrayI") jr_swizzleMethod:@selector(arrayByAddingObject:) withMethod:@selector(guard_arrayByAddingObject:) error:nil];
}

- (instancetype)initWithObjects_guard:(id *)objects count:(NSUInteger)cnt {
    NSUInteger newCnt = 0;
    for (NSUInteger i = 0; i < cnt; i++) {
        if (!objects[i]) {
            [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
            break;
        }
        newCnt++;
    }
    self = [self initWithObjects_guard:objects count:newCnt];
    return self;
}

- (id)guard_objectAtIndex:(NSUInteger)index {
    if (index >= [self count]) {
        // 收集堆栈，上报 Crash
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return nil;
    }
    return [self guard_objectAtIndex:index];
}

- (NSArray *)guard_arrayByAddingObject:(id)anObject {
    if (!anObject) {
        
        // 收集堆栈，上报 Crash
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return self;
    }
    return [self guard_arrayByAddingObject:anObject];
}

+ (void)warningDeveloper:(SEL)aSelector{
#if DEBUG
    
    NSString *selectorStr = NSStringFromSelector(aSelector);
    NSLog(@"PROTECTOR: -[%@ %@]", [self class], selectorStr);
    NSLog(@"PROTECTOR: unrecognized selector \"%@\" sent to instance: %p", selectorStr, self);
    NSLog(@"PROTECTOR: call stack: %@", [NSThread callStackSymbols]);
    NSAssert(NO, @"arr调用方法出错---[%@-%@]", [self class], selectorStr);
    // @throw @"方法找不到";
#endif
}
@end
