//
//  NSMutableArray+GHLCrashGuard.m
//  GHLCrashGuard
//
//  Created by 与佳期 on 2019/7/11.
//

#import "NSMutableArray+GHLCrashGuard.h"
#import "JRSwizzle.h"

@implementation NSMutableArray (GHLCrashGuard)

+ (void)load {
    
    [NSClassFromString(@"__NSArrayM") jr_swizzleMethod:@selector(addObject:) withMethod:@selector(guard_addObject:) error:nil];
    [NSClassFromString(@"__NSArrayM") jr_swizzleMethod:@selector(insertObject:atIndex:) withMethod:@selector(guard_insertObject:atIndex:) error:nil];
    [NSClassFromString(@"__NSArrayM") jr_swizzleMethod:@selector(removeObjectAtIndex:) withMethod:@selector(guard_removeObjectAtIndex:) error:nil];
    [NSClassFromString(@"__NSArrayM") jr_swizzleMethod:@selector(replaceObjectAtIndex:withObject:) withMethod:@selector(guard_replaceObjectAtIndex:withObject:) error:nil];
    [NSClassFromString(@"__NSArrayM") jr_swizzleMethod:@selector(objectAtIndex:) withMethod:@selector(guard_objectAtIndex:) error:nil];
}


- (void)guard_addObject:(id)anObject {
    if (!anObject) {
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return;
    }
    [self guard_addObject:anObject];
}

- (void)guard_insertObject:(id)anObject atIndex:(NSUInteger)index {
    if (index > [self count]) {
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return;
    }
    if (!anObject) {
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return;
    }
    [self guard_insertObject:anObject atIndex:index];
}

- (void)guard_removeObjectAtIndex:(NSUInteger)index {
    if (index >= [self count]) {
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return;
    }
    
    return [self guard_removeObjectAtIndex:index];
}
- (void)guard_replaceObjectAtIndex:(NSUInteger)index withObject:(id)anObject {
    if (index >= [self count]) {
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return;
    }
    if (!anObject) {
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return;
    }
    [self guard_replaceObjectAtIndex:index withObject:anObject];
}

- (id)guard_objectAtIndex:(NSUInteger)index {
    if (index >= [self count]) {
        // 收集堆栈，上报 Crash
        [[self class] warningDeveloper:NSSelectorFromString([NSString stringWithFormat:@"%s",__FUNCTION__])];
        return nil;
    }
    return [self guard_objectAtIndex:index];
}

+ (void)warningDeveloper:(SEL)aSelector{
#if DEBUG
    
    NSString *selectorStr = NSStringFromSelector(aSelector);
    NSLog(@"PROTECTOR: -[%@ %@]", [self class], selectorStr);
    NSLog(@"PROTECTOR: unrecognized selector \"%@\" sent to instance: %p", selectorStr, self);
    NSLog(@"PROTECTOR: call stack: %@", [NSThread callStackSymbols]);
    NSAssert(NO, @"marr调用出错---[%@-%@]", [self class], selectorStr);
    // @throw @"方法找不到";
#endif
}
@end
