//
//  NSObject+Selector.m
//  strongDemo
//
//  Created by sxk on 2020/8/3.
//  Copyright © 2020 sxk. All rights reserved.
//

#import "NSObject+Selector.h"
#import <objc/runtime.h>
@implementation NSObject (Selector)

+ (void)load
{
    Method method1 = class_getInstanceMethod(self, @selector(forwardingTargetForSelector:));
    Method method2 = class_getInstanceMethod(self, @selector(sxk_forwardingTargetForSelector:));
    method_exchangeImplementations(method1, method2);
}

- (id)sxk_forwardingTargetForSelector:(SEL)aSelector
{
    if ([self isCurrentClassInWhiteList]) {
        [[self class] warningDeveloper:aSelector];
        
        Class protectorCls = NSClassFromString(@"ProtectorClassName");
        if (!protectorCls){
            protectorCls = objc_allocateClassPair([NSObject class], "ProtectorClassName", 0);
            objc_registerClassPair(protectorCls);
        }
        
        if (![self isExistSelector:aSelector inClass:protectorCls]){
            //动态为一个类加载方法 使本次方法加载到ProtectorClassName上
            class_addMethod(protectorCls, aSelector, [self safeImplementation:aSelector],[NSStringFromSelector(aSelector) UTF8String]);
        }
        
        Class Protector = [protectorCls class];
        id instance = [[Protector alloc] init];
        return instance;
    } else {
        return nil;
    }
}

#pragma clang diagnostic pop

- (BOOL)isCurrentClassInWhiteList{
    NSArray *classNameArray = @[@"NSNull",@"NSString",@"NSArray",@"NSDictionary",@"NSURL"];
    for (NSString *className in classNameArray) {
        if ([self isKindOfClass:NSClassFromString(className)]) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)isExistSelector:(SEL)aSelector inClass:(Class)currentClass{
    BOOL isExist = NO;
    unsigned int methodCount = 0;
    Method *methods = class_copyMethodList(currentClass, &methodCount);
    for (int i = 0; i < methodCount; i++){
        Method temp = methods[i];
        SEL sel = method_getName(temp);
        NSString *methodName = NSStringFromSelector(sel);
        if ([methodName isEqualToString: NSStringFromSelector(aSelector)]){
            isExist = YES;
            break;
        }
    }
    return isExist;
}

- (IMP)safeImplementation:(SEL)aSelector{
    IMP imp = imp_implementationWithBlock(^(){
        NSLog(@"PROTECTOR: %@ Done", NSStringFromSelector(aSelector));
    });
    return imp;
}

+ (void)warningDeveloper:(SEL)aSelector{
#if DEBUG
    NSString *selectorStr = NSStringFromSelector(aSelector);
    NSLog(@"PROTECTOR: -[%@ %@]", [self class], selectorStr);
    NSLog(@"PROTECTOR: unrecognized selector \"%@\" sent to instance: %p", selectorStr, self);
    NSLog(@"PROTECTOR: call stack: %@", [NSThread callStackSymbols]);
    //NSAssert(NO, @"调用不存在的方法---[%@-%@]", [self class], selectorStr);
    // @throw @"方法找不到";
#endif
}
@end
